local _, T = ...
local L = T.L

local msq, msqGroups = nil, {}
if LibStub then
	msq = LibStub("Masque",true)
end
if msq then
	msqGroups = {
		FarmhandTools = msq:Group("Farmhand","Tools"),
		FarmhandSeeds = msq:Group("Farmhand","Seeds"),
	}
end

local Farmhand = CreateFrame("frame","Farmhand",UIParent)
Farmhand:RegisterEvent("PLAYER_LOGIN")
Farmhand:RegisterEvent("PLAYER_ENTERING_WORLD")
local function Farmhand_EventHandler(self, event, ...)
	if event == "PLAYER_LOGIN" then
		self:Initialize()
	elseif event == "PLAYER_ENTERING_WORLD" then
		self:UnregisterEvent("PLAYER_ENTERING_WORLD")
		self:RegisterEvent("BAG_UPDATE")
		self:RegisterEvent("ZONE_CHANGED")
		self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
		self:RegisterEvent("ZONE_CHANGED_INDOORS")
		self:RegisterEvent("MERCHANT_SHOW")
		self:RegisterEvent("MERCHANT_CLOSED")
	elseif event == "ZONE_CHANGED" or 
		   event == "ZONE_CHANGED_NEW_AREA" or 
		   event == "ZONE_CHANGED_INDOORS" or
		   event == "BAG_UPDATE" then
		self:Update()
	elseif event == "MERCHANT_SHOW" or event == "MERCHANT_CLOSED" then
		if self:IsShown() then
			self.SellMode = event == "MERCHANT_SHOW"
			self:UpdateBar(self.seeds)
		end
	elseif event == "PLAYER_REGEN_ENABLED" then
		self:UnregisterEvent("PLAYER_REGEN_ENABLED")
		if self.Initialized then
			self:Update()
		else
			self:Initialize()
		end	
	end
end
Farmhand:SetScript("OnEvent",Farmhand_EventHandler)
Farmhand:SetMovable(true)
Farmhand.L = L

Farmhand.tools = CreateFrame("Frame","FarmhandTools",Farmhand)
Farmhand.tools:SetPoint("Bottom",Farmhand,"Bottom",0,7)
Farmhand.tools:SetScale(1.5)
Farmhand.tools.items = {
	{id=89815}, --Master Plow
	{id=89880}, --Dented Shovel
	{id=80513}, --Vintage Bug Sprayer
	{id=79104}, --Rusty Watering Can
	{id="scanner"}, --Plant Scanner
}

Farmhand.seeds = CreateFrame("Frame","FarmhandSeeds",Farmhand)
Farmhand.seeds:SetPoint("Top",Farmhand,"Top",0,-15)
Farmhand.seeds:SetScale(1)
Farmhand.seeds.items = {
	{id=79102}, --Green Cabbage Seeds
	{id=89328}, --Jade Squash Seeds
	{id=80590}, --Juicycrunch Carrot Seeds
	{id=80592}, --Mogu Pumpkin Seeds
	{id=80594}, --Pink Turnip Seeds
	{id=80593}, --Red Blossom Leek Seeds
	{id=80591}, --Scallion Seeds
	{id=89329}, --Striped Melon Seeds
	{id=80595}, --White Turnip Seeds
	{id=89326}, --Witchberry Seeds
	{id=85219}, --Ominous Seed
	{id=85216}, --Enigma Seed
	{id=85217}, --Magebulb Seed
	{id=89202}, --Raptorleaf Seed
	{id=85215}, --Snakeroot Seed
	{id=89233}, --Songbell Seed
	{id=89197}, --Windshear Cactus Seed
	{id=91860}, --Stormwind Portal Shard
	{id=91864}, --Ironforge Portal Shard
	{id=91865}, --Darnassus Portal Shard
	{id=91866}, --Exodar Portal Shard
	{id=91850}, --Orgrimmar Portal Shard
	{id=91862}, --Undercity Portal Shard
	{id=91861}, --Thunder Bluff Portal Shard
	{id=91863}, --Silvermoon Portal Shard
}

local lock = CreateFrame("CheckButton","FarmhandToolLock",FarmhandTools)
lock:SetBackdrop({bgFile="Interface\\Buttons\\LockButton-Border"})
lock:SetNormalTexture("Interface\\Buttons\\LockButton-Unlocked-Up")
lock:SetPushedTexture("Interface\\Buttons\\LockButton-Unlocked-Down")
lock:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
lock:SetCheckedTexture("Interface\\Buttons\\LockButton-Locked-Up")
lock:SetSize(16,16)
lock:SetPoint("Right",FarmhandTools,"Left",0,0)
RegisterStateDriver(lock,"visibility","[mod:shift] show; hide")
lock:SetScript("OnClick",function(self)
	if self:GetChecked() == 1 then
		FarmhandData.ToolsLocked = true
		print(L["Tools are now |cFF00FF00Locked|r and |cFF00FF00will not|r be dropped when you leave Halfhill."])
	else
		FarmhandData.ToolsLocked = false
		print(L["Tools are now |cFFFF0000Unlocked|r and |cFFFF0000will|r be dropped when you leave Halfhill."])
	end
end)
Farmhand.lock = lock

function Farmhand:Initialize()
	if InCombatLockdown() then
		self:RegisterEvent("PLAYER_REGEN_ENABLED")
		return
	end
	if FarmhandData == nil then
		FarmhandData = {
			X = 0,
			Y = -250,
			ToolsLocked = false,
		}
	end

	Farmhand:SetPoint("Center",UIParent,"Center",FarmhandData.X,FarmhandData.Y)
	
	self.lock:SetChecked(FarmhandData.ToolsLocked == true)
	
	self:CreateButtons(self.tools)
	self:CreateButtons(self.seeds)

	self:Update()

	self.Initialized = true
end

function Farmhand:CreateButtons(bar)
	for i,item in ipairs(bar.items) do
	
		if item.id == "scanner" then
			item.btn = CreateFrame("Button", "FarmhandScanButton", FarmhandTools, "SecureActionButtonTemplate")
			item.btn:SetSize(32,32)
			item.btn.icon = item.btn:CreateTexture(item.btn:GetName().."Icon","BACKGROUND")
			item.btn.icon:SetTexture("Interface\\AddOns\\Farmhand\\RadarIcon.tga")
			item.btn.icon:SetAllPoints()
			item.btn.icon:Show()

			item.btn.count = item.btn:CreateFontString(item.btn:GetName().."Count","ARTWORK","NumberFontNormal")
			item.btn.count:SetJustifyH("RIGHT")
			item.btn.count:SetPoint("Bottom",0,2)
			
			item.btn:SetPushedTexture("Interface\\Buttons\\UI-Quickslot-Depress")
			item.btn:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square","ADD")

			item.btn:RegisterForClicks("AnyUp")
			item.btn:SetAttribute("type", "macro")
--			item.btn:SetAttribute("macrotext", L["CropScannerMacro"])
			item.btn:SetScript("PostClick",Farmhand.CropScannerPostClick)

			local ScanStages, MacroLength = {}, 0
			local lines = { strsplit("\n", gsub(L["CropScannerMacro"],"\r","")) }
			local stage = ""
			for i,line in ipairs(lines) do
				if strlen(stage) + strlen(line) + 1 > 1024 then
					ScanStages[#ScanStages+1] = stage
					stage = ""
				end
				stage = stage..line.."\n"
			end
			if strlen(stage) > 0 then
				ScanStages[#ScanStages+1] = stage
			end
			item.btn.ScanStages = ScanStages

			item.btn:SetAttribute("macrotext",ScanStages[1])
			
			item.btn.ScanStage = 1
			item.btn.count:SetText("1/"..#ScanStages)
			if #ScanStages == 1 then
				item.btn.count:Hide()
			end
			
		else
			item.btn = CreateFrame("Button", bar:GetName().."Button"..i, bar, "SecureActionButtonTemplate")
			item.btn:SetSize(32,32)

			item.btn.icon = item.btn:CreateTexture(item.btn:GetName().."Icon","BACKGROUND")
			item.btn.icon:SetAllPoints()
			item.btn.icon:Show()

			item.btn.count = item.btn:CreateFontString(item.btn:GetName().."Count","ARTWORK","NumberFontNormal")
			item.btn.count:SetJustifyH("RIGHT")
			item.btn.count:SetPoint("BottomRight",-2,2)

			item.btn:SetPushedTexture("Interface\\Buttons\\UI-Quickslot-Depress")
			item.btn:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square","ADD")
			
			item.btn:RegisterForClicks("AnyUp")
		end

		item.btn:SetScript("OnEnter",Farmhand.ButtonOnEnter)
		item.btn:SetScript("OnLeave",Farmhand.ButtonOnLeave)
		item.btn:SetScript("OnMouseDown",Farmhand.ButtonOnMouseDown)
		item.btn:SetScript("OnMouseUp",Farmhand.ButtonOnMouseUp)
		item.btn:SetScript("OnHide",Farmhand.ButtonOnHide)
		
		if msqGroups[bar:GetName()] then
			msqGroups[bar:GetName()]:AddButton(item.btn)
		end

	end
end

function Farmhand:CropScannerPostClick(button,down)
	if down or (button ~= "LeftButton" and button ~= "RightButton") then return end
	if self.ScanStage == #self.ScanStages then
		if UnitExists("target") then 
			SetRaidTarget("target",8)
		else 
			RaidNotice_AddMessage(RaidBossEmoteFrame,L["The crops are looking good!"], ChatTypeInfo["RAID_BOSS_EMOTE"])
		end
		self.ScanStage = 1
	else
		if UnitExists("target") then 
			SetRaidTarget("target",8)
			self.ScanStage = 1
		else 
			RaidNotice_AddMessage(RaidBossEmoteFrame,L["Nothing so far!"], ChatTypeInfo["RAID_BOSS_EMOTE"])
			self.ScanStage = self.ScanStage + 1
		end
	end
	self.count:SetText(self.ScanStage.."/"..#self.ScanStages)
	self:SetAttribute("macrotext",self.ScanStages[self.ScanStage])
end

function Farmhand:ButtonOnEnter()
	if Farmhand.SellMode and self:GetParent():GetName() == "FarmhandSeeds" then
		ShowContainerSellCursor(self.bag,self.slot)
	end
	GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
	if self:GetName() == "FarmhandScanButton" then
		GameTooltip:ClearLines()
		GameTooltip:AddLine(L["Crop Scanner"],0,255,0,false)
		GameTooltip:AddLine(L["Scan your farm for crops or soil that need attention."],255,255,255,true)
	else
		GameTooltip:SetBagItem(self.bag,self.slot)
	end
	GameTooltip:Show()
end

function Farmhand:ButtonOnLeave()
	if Farmhand.SellMode and self:GetParent():GetName() == "FarmhandSeeds" then
		ResetCursor()
	end
	GameTooltip_Hide()
end

function Farmhand:ButtonOnMouseDown(button)
	if IsShiftKeyDown() and button == "LeftButton" and not Farmhand.isMoving then
		Farmhand:StartMoving()
		Farmhand.isMoving = true
	elseif button == "RightButton" and Farmhand.isMoving then
		Farmhand:StopMovingOrSizing()
		Farmhand.isMoving = false
		FarmhandData.X, FarmhandData.Y = 0, -250
		Farmhand:SetPoint("Center",UIParent,"Center",0,-250)
	end
end

function Farmhand:ButtonOnMouseUp(button)
	if button == "LeftButton" and Farmhand.isMoving then
		Farmhand:StopMovingOrSizing()
		Farmhand.isMoving = false
		_,_,_,FarmhandData.X,FarmhandData.Y = Farmhand:GetPoint(1)
   end
end

function Farmhand:ButtonOnHide()
	if Farmhand.isMoving then
		Farmhand:StopMovingOrSizing()
		Farmhand.isMoving = false
		Farmhand:SetPoint("Center", UIParent, "Center", FarmhandData.X, FarmhandData.Y)
	end
end

function Farmhand:UpdateBar(bar)
	local last
	local shown = 0
	local ButtonSpacing = 6

	for _,item in ipairs(bar.items) do
		local ItemCount = GetItemCount(item.id)
		if ItemCount > 0 or item.id == "scanner" then
			if shown % 8 == 0 then
				local row = math.floor(shown/8)
				item.btn:SetPoint("TopLeft",bar,"TopLeft", ButtonSpacing/2, -(ButtonSpacing * .5 ) - row*item.btn:GetHeight() - row*ButtonSpacing)
				last = item.btn
			else
				item.btn:SetPoint("TopLeft",last,"TopRight",ButtonSpacing,0)
				last = item.btn
			end
			if IsConsumableItem(item.id) then
				local count = _G[item.btn:GetName().."Count"]
				if ItemCount > 999 then
					count:SetText("***")
				else
					count:SetText(ItemCount)
				end
			end
			local bag, slot = self:FindItemInBags(item.id)
			if bag and slot then
				local icon = GetContainerItemInfo(bag,slot)
				item.btn.bag = bag
				item.btn.slot = slot
				item.btn.icon:SetTexture(icon)
				if bar:GetName() == "FarmhandSeeds" then
					item.btn:SetAttribute("type", "macro")
					item.btn:SetAttribute("macrotext","/targetexact [noexists] "..L["Tilled Soil"].."\n/use "..bag.." "..slot)
				else
					item.btn:SetAttribute("type", "item")
					item.btn:SetAttribute("item", bag.." "..slot)
				end
			end
			item.btn:Show()
			shown = shown + 1
		else
			item.btn:Hide()
		end
	end

	if msqGroups[bar:GetName()] then msqGroups[bar:GetName()]:ReSkin() end
	
	if last ~= nil then
		bar:SetWidth(math.min(8,shown) * (last:GetWidth() + ButtonSpacing))
		bar:SetHeight(math.ceil(shown/8) * (last:GetHeight() + ButtonSpacing))
		bar:Show()
	else
		bar:Hide()
	end
end

function Farmhand:Update()

	if InCombatLockdown() then
		self:RegisterEvent("PLAYER_REGEN_ENABLED")
		return
	end

	local Zone, SubZone = GetZoneText(), GetSubZoneText()

	if Zone == "" and SubZone == "" then 
		return
	end

	if self.InHalfhill and SubZone ~= L["Sunsong Ranch"] and SubZone ~= L["The Halfhill Market"] and SubZone ~= L["Halfhill"] and Zone ~= L["Halfhill"] then
		if FarmhandData.ToolsLocked == true then
			print(L["Leaving Halfhill."].." "..L["Tools are Locked."])
		else
			ClearCursor()
			for i,item in ipairs(self.tools.items) do
				local bag, slot = self:FindItemInBags(item.id)
				if bag and slot then
					PickupContainerItem(bag,slot)
					if CursorHasItem() then
						local _, id, link = GetCursorInfo()
						if id == item.id then
							print(L["Leaving Halfhill."].." "..L["Dropping"].." "..link..".")
							DeleteCursorItem()
						end
					end
				end
			end
		end
	end

	self.InHalfhill = SubZone == L["Sunsong Ranch"] or SubZone == L["The Halfhill Market"] or SubZone == L["Halfhill"] or Zone == L["Halfhill"]
	
	if SubZone == L["Sunsong Ranch"] or SubZone == L["The Halfhill Market"] then 
		self:UpdateBar(self.tools)
		self:UpdateBar(self.seeds)
			
		local th = (self.tools:IsShown() and self.tools:GetHeight()*self.tools:GetScale() or 0)
		local sh = (self.seeds:IsShown() and self.seeds:GetHeight()*self.seeds:GetScale() or 0)
		self:SetHeight(24 + th + sh)
		
		local tw = (self.tools:IsShown() and self.tools:GetWidth()*self.tools:GetScale() or 0)
		local sw = (self.seeds:IsShown() and self.seeds:GetWidth()*self.seeds:GetScale() or 0)
		self:SetWidth(24 + math.max(tw,sw))
		
		self:Show()
	elseif self:IsShown() then
		self:Hide()
	end

end

function Farmhand:FindItemInBags(item)
	for container = 0, NUM_BAG_SLOTS do
		local numSlots = GetContainerNumSlots(container)
		if numSlots then
			for slot = 1, numSlots do
				if item == GetContainerItemID(container, slot) then
					return container, slot
				end
			end
		end
	end
end
